@@ -106,6 +106,7 @@ namespace canvas
     addStyle("preserveAspectRatio", "", &Image::setPreserveAspectRatio);
     addStyle("slice", "", &Image::setSlice);
     addStyle("slice-width", "", &Image::setSliceWidth);
+    addStyle("opacity", "numeric", &Image::setOpacity);
 
     osgDB::Registry* reg = osgDB::Registry::instance();
     if( !reg->getReaderWriterForExtension("png") )
@@ -131,11 +132,11 @@ namespace canvas
     stateSet->setDataVariance(osg::Object::STATIC);
 
     // allocate arrays for the image
-    _vertices = new osg::Vec3Array(4);
+    _vertices = new osg::Vec3Array(6);
     _vertices->setDataVariance(osg::Object::DYNAMIC);
     _geom->setVertexArray(_vertices);
 
-    _texCoords = new osg::Vec2Array(4);
+    _texCoords = new osg::Vec2Array(6);
     _texCoords->setDataVariance(osg::Object::DYNAMIC);
     _geom->setTexCoordArray(0, _texCoords, osg::Array::BIND_PER_VERTEX);
 
@@ -143,8 +144,8 @@ namespace canvas
     _colors->setDataVariance(osg::Object::DYNAMIC);
     _geom->setColorArray(_colors, osg::Array::BIND_OVERALL);
 
-    _prim = new osg::DrawArrays(osg::PrimitiveSet::QUADS);
-    _prim->set(osg::PrimitiveSet::QUADS, 0, 4);
+    _prim = new osg::DrawArrays(osg::PrimitiveSet::TRIANGLES);
+    _prim->set(osg::PrimitiveSet::TRIANGLES, 0, 6);
     _prim->setDataVariance(osg::Object::DYNAMIC);
     _geom->addPrimitiveSet(_prim);
 
@@ -180,13 +181,7 @@ namespace canvas
         && child->getNameString() == "visible"
         && child->getBoolValue() )
     {
-      CullCallback* cb =
-#if OSG_VERSION_LESS_THAN(3,3,2)
-        static_cast<CullCallback*>
-#else
-        dynamic_cast<CullCallback*>
-#endif
-        ( _geom->getCullCallback() );
+      CullCallback* cb = dynamic_cast<CullCallback*>( _geom->getCullCallback() );
 
       if( cb )
         cb->cullNextFrame();
@@ -255,12 +250,19 @@ namespace canvas
     setFill(color);
   }
 
-    //----------------------------------------------------------------------------
-    void Image::setFill(const osg::Vec4& color)
-    {
-        _colors->front() = color;
-        _colors->dirty();
-    }
+  //----------------------------------------------------------------------------
+  void Image::setFill(const osg::Vec4& color)
+  {
+      _colors->front() = color;
+      _colors->dirty();
+  }
+
+  //----------------------------------------------------------------------------
+  void Image::setOpacity(float opacity)
+  {
+      _colors->front().a() = opacity;
+      _colors->dirty();
+  }
 
 
   //----------------------------------------------------------------------------
@@ -383,7 +385,7 @@ namespace canvas
 
     if( _attributes_dirty & DEST_SIZE )
     {
-      size_t num_vertices = (_slice.isValid() ? (fill ? 9 : 8) : 1) * 4;
+      size_t num_vertices = (_slice.isValid() ? (fill ? 9 : 8) : 1) * 6;
 
       if( num_vertices != _prim->getNumPrimitives() )
       {
@@ -762,21 +764,25 @@ namespace canvas
   //----------------------------------------------------------------------------
   void Image::setQuad(size_t index, const SGVec2f& tl, const SGVec2f& br)
   {
-    int i = index * 4;
-    (*_vertices)[i + 0].set(tl.x(), tl.y(), 0);
-    (*_vertices)[i + 1].set(br.x(), tl.y(), 0);
-    (*_vertices)[i + 2].set(br.x(), br.y(), 0);
-    (*_vertices)[i + 3].set(tl.x(), br.y(), 0);
+    int i = index * 6;
+    (*_vertices)[i + 0].set(tl.x(), br.y(), 0.0f);
+    (*_vertices)[i + 1].set(tl.x(), tl.y(), 0.0f);
+    (*_vertices)[i + 2].set(br.x(), tl.y(), 0.0f);
+    (*_vertices)[i + 3].set(tl.x(), br.y(), 0.0f);
+    (*_vertices)[i + 4].set(br.x(), tl.y(), 0.0f);
+    (*_vertices)[i + 5].set(br.x(), br.y(), 0.0f);
   }
 
   //----------------------------------------------------------------------------
   void Image::setQuadUV(size_t index, const SGVec2f& tl, const SGVec2f& br)
   {
-    int i = index * 4;
-    (*_texCoords)[i + 0].set(tl.x(), tl.y());
-    (*_texCoords)[i + 1].set(br.x(), tl.y());
-    (*_texCoords)[i + 2].set(br.x(), br.y());
+    int i = index * 6;
+    (*_texCoords)[i + 0].set(tl.x(), br.y());
+    (*_texCoords)[i + 1].set(tl.x(), tl.y());
+    (*_texCoords)[i + 2].set(br.x(), tl.y());
     (*_texCoords)[i + 3].set(tl.x(), br.y());
+    (*_texCoords)[i + 4].set(br.x(), tl.y());
+    (*_texCoords)[i + 5].set(br.x(), br.y());
   }
 
   //----------------------------------------------------------------------------
