@@ -518,6 +518,10 @@ size_t Client::requestReadCallback(char *ptr, size_t size, size_t nmemb, void *u
 
 bool isRedirectStatus(int code)
 {
+    if (code == 304) {
+        return false;
+    }
+
     return ((code >= 300) && (code < 400));
 }
 
@@ -542,12 +546,14 @@ size_t Client::requestHeaderCallback(char *rawBuffer, size_t size, size_t nitems
   }
 
   if (h.empty()) {
-      // got a 100-continue reponse; restart
+      // end of headers
+      // got a 100-continue response; restart
       if (req->responseCode() == 100) {
           req->setReadyState(HTTP::Request::OPENED);
           return byteSize;
       }
 
+
     req->responseHeadersComplete();
     return byteSize;
   }
