@@ -1,30 +1,15 @@
-// Copyright (C) 2016  James Turner - <zakalawe@mac.com>
-//
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
-//
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software
-// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-//
+// SPDX-License-Identifier: LGPL-2.1-or-later
+// SPDX-FileCopyrightText: Copyright (C) 2016 James Turner <james@flightgear.org>
 
 #include <simgear_config.h>
 
 #include "untar.hxx"
 
-#include <cstdlib>
-#include <cassert>
-#include <stdint.h>
-#include <cstring>
-#include <cstddef>
 #include <algorithm>
+#include <cstddef>
+#include <cstdlib>
+#include <cstring>
+#include <stdint.h>
 
 #include <zlib.h>
 
@@ -39,6 +24,7 @@
 #include <simgear/structure/exception.hxx>
 
 #include "ArchiveExtractor_private.hxx"
+#include "simgear/debug/debug_types.h"
 
 namespace simgear
 {
@@ -67,7 +53,7 @@ typedef struct
     const size_t TAR_HEADER_BLOCK_SIZE = 512;
 
 #define TMAGIC   "ustar"        /* ustar and a null */
-#define TMAGLEN  5              // 5, not 6, becuase some files use 'ustar '
+#define TMAGLEN  5              // 5, not 6, because some files use 'ustar '
 #define TVERSION "00"           /* 00 and no null */
 #define TVERSLEN 2
 
@@ -82,13 +68,15 @@ typedef struct
 #define FIFOTYPE '6'            /* FIFO special */
 #define CONTTYPE '7'            /* reserved */
 
+#define GNU_LONGNAME 'L' /* GNU tar systeem for longer file-names */
+
     const char PAX_GLOBAL_HEADER = 'g';
     const char PAX_FILE_ATTRIBUTES = 'x';
 
 
     ///////////////////////////////////////////////////////////////////////////////////////////////////
 
-    const int ZLIB_DECOMPRESS_BUFFER_SIZE = 32 * 1024;
+    const int ZLIB_DECOMPRESS_BUFFER_SIZE = 1024 * 1024;
     const int ZLIB_INFLATE_WINDOW_BITS = MAX_WBITS;
     const int ZLIB_DECODE_GZIP_HEADER = 16;
 
@@ -105,13 +93,16 @@ typedef struct
 
         size_t bytesRemaining;
         std::unique_ptr<SGFile> currentFile;
-        size_t currentFileSize;
+        size_t currentFileSize; ///< if we're in READING_GNU_LONGNAME, this is
+                                ///< the name length
 
         uint8_t* headerPtr;
         bool skipCurrentEntry = false;
         std::string paxAttributes;
         std::string paxPathName;
 
+        std::string gnuLongName;
+
         TarExtractorPrivate(ArchiveExtractor* o) : ArchiveExtractorPrivate(o)
         {
             setState(TarExtractorPrivate::READING_HEADER);
@@ -139,6 +130,7 @@ typedef struct
             if (state == READING_FILE) {
                 if (currentFile) {
                     currentFile->close();
+                    writeDirHashEntry();
                     currentFile.reset();
                 }
                 readPaddingIfRequired();
@@ -158,6 +150,8 @@ typedef struct
                 readPaddingIfRequired();
             } else if (state == READING_PADDING) {
                 setState(READING_HEADER);
+            } else if (state == READING_GNU_LONGNAME) {
+              readPaddingIfRequired();
             }
         }
 
@@ -168,6 +162,12 @@ typedef struct
                 headerPtr = headerBytes;
             }
 
+            if (newState >= ERROR_STATE) {
+                SG_LOG(SG_IO, SG_WARN, "ArchiveExtract entered error state:" << static_cast<int>(newState));
+                currentFile->close();
+                currentFile.reset();
+            }
+
             state = newState;
         }
 
@@ -182,6 +182,26 @@ typedef struct
             // no-op for tar files, we process everything greedily
         }
 
+        void writeDirHashEntry()
+        {
+            if (!doCreateDirHashes()) {
+                return;
+            }
+
+            std::string hashBytes((char*)sha1_result(&hashState), HASH_LENGTH);
+            SGPath path = currentFile->get_path();
+
+            // force a stat call
+            path.set_cached(false);
+            path.set_cached(true);
+
+            // this code has to match the main version in HTTPRepository.cxx
+            SGPath cachePath = path.dirPath() / ".dirhash";
+            sg_ofstream stream(cachePath, std::ios::out | std::ios::app | std::ios::binary);
+            stream << path.utf8Str() << "*" << path.modTime() << "*"
+                   << path.sizeInBytes() << "*" << strutils::encodeHex(hashBytes) << "\n";
+        }
+
         void processHeader()
         {
             if (headerIsAllZeros()) {
@@ -199,15 +219,49 @@ typedef struct
                 return;
             }
 
+            if (header.typeflag == GNU_LONGNAME) {
+              currentFileSize = ::strtol(header.size, NULL, 8);
+              bytesRemaining = currentFileSize;
+              gnuLongName.clear();
+              setState(READING_GNU_LONGNAME);
+              return;
+            }
+
             skipCurrentEntry = false;
-            std::string tarPath = std::string(header.prefix) + std::string(header.fileName);
+
+            // careful handling here for tar compressors which don't use PAX path
+            // attribute to handle filenames longer than 100 bytes, or exactly 100 bytes.
+            // longer than 100 bytes use the prefix, but we need to add a path seperator.
+            // exactly 100 bytes don't use prefix, but there is no trailing NULL.
+            // https://sourceforge.net/p/flightgear/codetickets/2953/
+
+            // GNU Long-name handling also needs its own case. <sigh
+            std::string tarPath;
             if (!paxPathName.empty()) {
                 tarPath = paxPathName;
                 paxPathName.clear(); // clear for next file
+            } else if (!gnuLongName.empty()) {
+              tarPath = gnuLongName;
+              gnuLongName.clear(); // clear for next file
+            } else if (strlen(header.prefix) > 0) {
+              tarPath = std::string(header.prefix) + "/" +
+                        std::string(header.fileName);
+            } else {
+              // handle fileNames which are exactly 100 bytes long.
+              const auto len = strnlen(header.fileName, 100);
+              tarPath = std::string(header.fileName, len);
             }
 
-            if (!isSafePath(tarPath)) {
-                SG_LOG(SG_IO, SG_WARN, "unsafe tar path, skipping::" << tarPath);
+            if (doRemoveTopmostDir()) {
+                const auto firstDir = tarPath.find('/');
+                tarPath.erase(0, firstDir + 1);
+                if (tarPath.empty()) {
+                    skipCurrentEntry = true;
+                }
+            }
+
+            if (!skipCurrentEntry && !isSafePath(tarPath)) {
+                SG_LOG(SG_IO, SG_WARN, "unsafe tar path, skipping:" << tarPath);
                 skipCurrentEntry = true;
             }
 
@@ -227,13 +281,25 @@ typedef struct
                 }
                 setState(READING_HEADER);
             } else if ((header.typeflag == REGTYPE) || (header.typeflag == AREGTYPE)) {
+                // create enclosing directory heirarchy as required
+                Dir parentDir(p.dir());
+                if (!parentDir.exists()) {
+                    bool ok = parentDir.create(0755);
+                    if (!ok) {
+                        throw sg_io_exception("failed to create directory heirarchy for extraction", p);
+                    }
+                }
+
                 currentFileSize = ::strtol(header.size, NULL, 8);
                 bytesRemaining = currentFileSize;
                 if (!skipCurrentEntry) {
                     currentFile.reset(new SGBinaryFile(p));
                     currentFile->open(SG_IO_OUT);
+                    sha1_init(&hashState);
                 }
                 setState(READING_FILE);
+                mostRecentPath = p;
+                SG_LOG(SG_IO, SG_DEBUG, "Will extract" << p);
             } else if (header.typeflag == PAX_GLOBAL_HEADER) {
                 setState(READING_PAX_GLOBAL_ATTRIBUTES);
                 currentFileSize = ::strtol(header.size, NULL, 8);
@@ -256,31 +322,35 @@ typedef struct
 
         void processBytes(const char* bytes, size_t count)
         {
-            if ((state >= ERROR_STATE) || (state == END_OF_ARCHIVE)) {
-                return;
-            }
-
-            size_t curBytes = std::min(bytesRemaining, count);
-            if (state == READING_FILE) {
-                if (currentFile) {
-                    currentFile->write(bytes, curBytes);
+            while (count > 0) {
+                if ((state >= ERROR_STATE) || (state == END_OF_ARCHIVE)) {
+                    return;
                 }
-                bytesRemaining -= curBytes;
-            } else if ((state == READING_HEADER) || (state == PRE_END_OF_ARCHVE) || (state == END_OF_ARCHIVE)) {
-                memcpy(headerPtr, bytes, curBytes);
-                bytesRemaining -= curBytes;
-                headerPtr += curBytes;
-            } else if (state == READING_PADDING) {
-                bytesRemaining -= curBytes;
-            } else if ((state == READING_PAX_FILE_ATTRIBUTES) || (state == READING_PAX_GLOBAL_ATTRIBUTES)) {
-                bytesRemaining -= curBytes;
-                paxAttributes.append(bytes, curBytes);
-            }
 
-            checkEndOfState();
-            if (count > curBytes) {
-                // recurse with unprocessed bytes
-                processBytes(bytes + curBytes, count - curBytes);
+                size_t curBytes = std::min(bytesRemaining, count);
+                if (state == READING_FILE) {
+                    if (currentFile) {
+                        currentFile->write(bytes, curBytes);
+                        sha1_write(&hashState, bytes, curBytes);
+                    }
+                    bytesRemaining -= curBytes;
+                } else if ((state == READING_HEADER) || (state == PRE_END_OF_ARCHVE) || (state == END_OF_ARCHIVE)) {
+                    memcpy(headerPtr, bytes, curBytes);
+                    bytesRemaining -= curBytes;
+                    headerPtr += curBytes;
+                } else if (state == READING_PADDING) {
+                    bytesRemaining -= curBytes;
+                } else if ((state == READING_PAX_FILE_ATTRIBUTES) || (state == READING_PAX_GLOBAL_ATTRIBUTES)) {
+                    bytesRemaining -= curBytes;
+                    paxAttributes.append(bytes, curBytes);
+                } else if (state == READING_GNU_LONGNAME) {
+                    bytesRemaining -= curBytes;
+                    gnuLongName.append(bytes, curBytes);
+                }
+
+                bytes += curBytes;
+                count -= curBytes;
+                checkEndOfState();
             }
         }
 
@@ -304,7 +374,7 @@ typedef struct
                 auto firstSpace = paxAttributes.find(' ', lineStart);
                 auto firstEq = paxAttributes.find('=', lineStart);
                 if ((firstEq == std::string::npos) || (firstSpace == std::string::npos)) {
-                    SG_LOG(SG_IO, SG_WARN, "Malfroemd PAX attributes in tarfile");
+                    SG_LOG(SG_IO, SG_WARN, "Malformed PAX attributes in tarfile");
                     break;
                 }
 
@@ -326,6 +396,7 @@ typedef struct
             if (!isGlobalAttr && (attrName == "path")) {
                 // data is UTF-8 encoded path name
                 paxPathName = data;
+                SG_LOG(SG_IO, SG_DEBUG, "Got PAX path attribute:" << data);
             }
         }
 };
@@ -427,11 +498,12 @@ public:
     XZTarExtractor(ArchiveExtractor* outer) : TarExtractorPrivate(outer)
     {
         _xzStream = LZMA_STREAM_INIT;
-        _outputBuffer = (uint8_t*)malloc(ZLIB_DECOMPRESS_BUFFER_SIZE);
+        _outputBuffer = (uint8_t*)malloc(ZLIB_DECOMPRESS_BUFFER_SIZE * 8);
 
 
         auto ret = lzma_stream_decoder(&_xzStream, UINT64_MAX, LZMA_TELL_ANY_CHECK);
         if (ret != LZMA_OK) {
+            SG_LOG(SG_IO, SG_WARN, "XZTarExtractor: lzma_stream_decoder failed to create:" << ret);
             setState(BAD_ARCHIVE);
             return;
         }
@@ -466,6 +538,7 @@ public:
                 setState(END_OF_ARCHIVE);
                 break;
             } else if (ret != LZMA_OK) {
+                SG_LOG(SG_IO, SG_WARN, "XZTarExtractor:extractBytes lzma_code returned error:" << ret);
                 setState(BAD_ARCHIVE);
                 break;
             }
@@ -476,6 +549,7 @@ public:
     {
         const auto ret = lzma_code(&_xzStream, LZMA_FINISH);
         if (ret != LZMA_STREAM_END) {
+            SG_LOG(SG_IO, SG_WARN, "XZTarExtractor:flush lzma_code returned error:" << ret);
             setState(BAD_ARCHIVE);
         }
     }
@@ -511,7 +585,7 @@ public:
 
 	void extractBytes(const uint8_t* bytes, size_t count) override
 	{
-		// becuase the .zip central directory is at the end of the file,
+		// because the .zip central directory is at the end of the file,
 		// we have no choice but to simply buffer bytes here until flush()
 		// is called
 		m_buffer.append((const char*) bytes, count);
@@ -606,8 +680,9 @@ public:
 		sg_ofstream outFile;
 		bool eof = false;
 		SGPath path = extractRootPath() / name;
+        mostRecentPath = name;
 
-		// create enclosing directory heirarchy as required
+        // create enclosing directory heirarchy as required
 		Dir parentDir(path.dir());
 		if (!parentDir.exists()) {
 			bool ok = parentDir.create(0755);
@@ -733,10 +808,11 @@ ArchiveExtractor::DetermineResult ArchiveExtractor::determineType(const uint8_t*
     }
 
     auto r = isTarData(bytes, count);
-    if ((r == TarData) || (r == InsufficientData) || (r == GZData))
+    if ((r == TarData) || (r == InsufficientData) || (r == GZData)) {
         return r;
+    }
 
-	return Invalid;
+    return Invalid;
 }
 
 
@@ -813,4 +889,24 @@ auto ArchiveExtractor::filterPath(std::string& pathToExtract)
     return Accepted;
 }
 
+void ArchiveExtractor::setCreateDirHashEntries(bool doCreate)
+{
+    _doCreateDirHashes = doCreate;
+}
+
+void ArchiveExtractor::setRemoveTopmostDirectory(bool doRemove)
+{
+    _removeTopmostDir = doRemove;
+}
+
+SGPath ArchiveExtractor::mostRecentExtractedPath() const
+{
+    if (!d) {
+        return SGPath();
+    }
+
+    return d->mostRecentPath;
+}
+
+
 } // of simgear
