@@ -15,8 +15,9 @@
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
 
-#include <simgear_config.h>
+#include "simgear/debug/debug_types.h"
 #include <simgear/package/Catalog.hxx>
+#include <simgear_config.h>
 
 #include <algorithm>
 #include <cassert>
@@ -143,9 +144,15 @@ protected:
         Dir d(m_owner->installRoot());
         SGPath p = d.file("catalog.xml");
         sg_ofstream f(p, std::ios::out | std::ios::trunc);
-        f.write(m_buffer.data(), m_buffer.size());
+        const auto sz = m_buffer.size();
+        f.write(m_buffer.data(), sz);
         f.close();
 
+        if (f.fail()) {
+            m_owner->refreshComplete(Delegate::FAIL_FILESYSTEM);
+            return;
+        }
+
         time(&m_owner->m_retrievedTime);
         m_owner->writeTimestamp();
         m_owner->refreshComplete(Delegate::STATUS_REFRESHED);
@@ -153,6 +160,13 @@ protected:
 
     void onFail() override
     {
+        const auto root = m_owner->root();
+        if (!root->isOnline()) {
+            // if we get a network failure, but we're offline, stay quiet
+            m_owner->refreshComplete(Delegate::FAIL_OFFLINE);
+            return;
+        }
+
         // network level failure
         SG_LOG(SG_GENERAL, SG_WARN, "catalog network failure for:" << m_owner->url());
         m_owner->refreshComplete(Delegate::FAIL_DOWNLOAD);
@@ -252,7 +266,7 @@ bool Catalog::uninstall()
     bool atLeastOneFailure = false;
 
     try {
-        // clean uninstall of each airacft / package in turn. This is
+        // clean uninstall of each aircraft / package in turn. This is
         // slightly overkill since we then nuke the entire catalog
         // directory anyway
         for (PackageRef p : installedPackages()) {
@@ -289,21 +303,30 @@ bool Catalog::removeDirectory()
     return d.remove(true /* recursive */);
 }
 
-PackageList const&
-Catalog::packages() const
+PackageList
+Catalog::packages(Type ty) const
 {
-  return m_packages;
+    if (ty == AnyPackageType)
+        return m_packages;
+
+    PackageList r;
+    std::copy_if(m_packages.begin(), m_packages.end(),
+                 std::back_inserter(r),
+                 [ty](const PackageRef& p) {
+                     return p->type() == ty;
+                 });
+    return r;
 }
 
 PackageList
 Catalog::packagesMatching(const SGPropertyNode* aFilter) const
 {
     PackageList r;
-    for (auto p : m_packages) {
-        if (p->matches(aFilter)) {
-            r.push_back(p);
-        }
-    }
+    std::copy_if(m_packages.begin(), m_packages.end(),
+                 std::back_inserter(r),
+                 [aFilter](const PackageRef& p) {
+                     return p->matches(aFilter);
+                 });
     return r;
 }
 
@@ -311,27 +334,25 @@ PackageList
 Catalog::packagesNeedingUpdate() const
 {
     PackageList r;
-    for (auto p : m_packages) {
-        if (!p->isInstalled()) {
-            continue;
-        }
-
-        if (p->install()->hasUpdate()) {
-            r.push_back(p);
-        }
-    }
+    std::copy_if(m_packages.begin(), m_packages.end(),
+                 std::back_inserter(r),
+                 [](const PackageRef& p) {
+                     return p->isInstalled() && p->install()->hasUpdate();
+                 });
     return r;
 }
 
 PackageList
-Catalog::installedPackages() const
+Catalog::installedPackages(Type ty) const
 {
   PackageList r;
-  for (auto p : m_packages) {
-    if (p->isInstalled()) {
-      r.push_back(p);
-    }
-  }
+  std::copy_if(m_packages.begin(), m_packages.end(),
+               std::back_inserter(r),
+               [ty](const PackageRef& p) {
+                   if (ty != AnyPackageType && (p->type() != ty))
+                       return false;
+                   return p->isInstalled();
+               });
   return r;
 }
 
@@ -342,6 +363,11 @@ void Catalog::refresh()
         return;
     }
 
+    if (!m_root->isOnline()) {
+        SG_LOG(SG_NETWORK, SG_INFO, "Catalog refresh skipped in offline mode.");
+        return;
+    }
+
     Downloader* dl = new Downloader(this, url());
     m_refreshRequest = dl;
     // will update status to IN_PROGRESS
@@ -372,8 +398,8 @@ void Catalog::parseProps(const SGPropertyNode* aProps)
     int nChildren = aProps->nChildren();
     for (int i = 0; i < nChildren; i++) {
         const SGPropertyNode* pkgProps = aProps->getChild(i);
-        if (strcmp(pkgProps->getName(), "package") == 0) {
-            // can't use getPackageById here becuase the variant dict isn't
+        if (pkgProps->getNameString() == "package") {
+            // can't use getPackageById here because the variant dict isn't
             // built yet. Instead we need to look at m_packages directly.
 
             PackageList::iterator pit = std::find_if(m_packages.begin(), m_packages.end(),
@@ -395,13 +421,13 @@ void Catalog::parseProps(const SGPropertyNode* aProps)
                 m_variantDict[*it] = p.ptr();
             }
         } else {
-            SGPropertyNode* c = m_props->getChild(pkgProps->getName(), pkgProps->getIndex(), true);
+            SGPropertyNode* c = m_props->getChild(pkgProps->getNameString().c_str(), pkgProps->getIndex(), true);
             copyProperties(pkgProps, c);
         }
     } // of children iteration
 
     if (!orphans.empty()) {
-        SG_LOG(SG_GENERAL, SG_WARN, "have orphan packages: will become inaccesible");
+        SG_LOG(SG_GENERAL, SG_WARN, "have orphan packages: will become inaccessible");
         std::set<PackageRef>::iterator it;
         for (it = orphans.begin(); it != orphans.end(); ++it) {
             SG_LOG(SG_GENERAL, SG_WARN, "\torphan package:" << (*it)->qualifiedId());
@@ -415,7 +441,7 @@ void Catalog::parseProps(const SGPropertyNode* aProps)
         if (m_url != m_props->getStringValue("url")) {
             // this effectively allows packages to migrate to new locations,
             // although if we're going to rely on that feature we should
-            // maybe formalise it!
+            // maybe formalize it!
             SG_LOG(SG_GENERAL, SG_WARN, "package downloaded from:" << m_url
                    << " is now at: " << m_props->getStringValue("url"));
         }
@@ -619,16 +645,10 @@ void Catalog::setUserEnabled(bool b)
 
 void Catalog::processAlternate(SGPropertyNode_ptr alt)
 {
-    std::string altId;
-    const auto idPtr = alt->getStringValue("id");
-    if (idPtr) {
-        altId = std::string(idPtr);
-    }
+    m_refreshRequest.reset();
+    std::string altId = alt->getStringValue("id");
 
-    std::string altUrl;
-    if (alt->getStringValue("url")) {
-        altUrl = std::string(alt->getStringValue("url"));
-    }
+    std::string altUrl = alt->getStringValue("url");
 
     CatalogRef existing;
     if (!altId.empty()) {
@@ -646,7 +666,7 @@ void Catalog::processAlternate(SGPropertyNode_ptr alt)
     // we have an alternate ID, and it's different from our ID, so let's
     // define a new catalog
     if (!altId.empty()) {
-      // don't auto-re-add Catalogs the user has explicilty rmeoved, that would
+      // don't auto-re-add Catalogs the user has explicitly removed, that would
       // suck
       const auto removedByUser = root()->explicitlyRemovedCatalogs();
       auto it = std::find(removedByUser.begin(), removedByUser.end(), altId);
@@ -694,8 +714,8 @@ void Catalog::processAlternate(SGPropertyNode_ptr alt)
 
     SG_LOG(SG_GENERAL, SG_INFO, "Migrating catalog " << id() << " to new URL:" << altUrl);
     setUrl(altUrl);
-    Downloader* dl = new Downloader(this, altUrl);
-    root()->makeHTTPRequest(dl);
+    m_refreshRequest = new Downloader(this, altUrl);
+    root()->makeHTTPRequest(m_refreshRequest);
 }
 
 int Catalog::markPackagesForInstallation(const string_list &packageIds) {
@@ -718,6 +738,32 @@ int Catalog::markPackagesForInstallation(const string_list &packageIds) {
 
 CatalogRef Catalog::migratedFrom() const { return m_migratedFrom; }
 
+PackageList Catalog::packagesProviding(const Type inferredType, const std::string& directory, const std::string& subpath) const
+{
+    PackageList p;
+    copy_if(m_packages.begin(), m_packages.end(), std::back_inserter(p),
+            [inferredType, &directory, &subpath](const PackageRef& pkg) {
+                // if we detected a package type, it needs to match, so the resulting
+                // path matches as well
+                if (inferredType != AnyPackageType) {
+                    if (inferredType != pkg->type()) {
+                        return false;
+                    }
+                }
+
+                if (directory != pkg->dirName())
+                    return false;
+
+                if (subpath.empty()) {
+                    return true; // we're done, success
+                }
+
+                return pkg->doesProvidePath(subpath);
+            });
+    return p;
+}
+
+
 } // of namespace pkg
 
 } // of namespace simgear
