@@ -15,6 +15,7 @@
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
 
+#include "simgear/debug/debug_types.h"
 #include <simgear_config.h>
 
 #include <simgear/package/Package.hxx>
@@ -29,12 +30,33 @@
 #include <simgear/package/Install.hxx>
 #include <simgear/package/Root.hxx>
 
+namespace {
+
+const string_list static_typeNames = {
+    "aircraft",
+    "ai-model",
+    "add-on",
+    "library"};
+
+} // namespace
+
 namespace simgear {
 
 namespace pkg {
 
+Type typeFromString(const std::string& s)
+{
+    const auto l = strutils::lowercase(s);
+    const auto it = std::find(static_typeNames.begin(), static_typeNames.end(), l);
+    if (it != static_typeNames.end()) {
+        return static_cast<Type>(std::distance(static_typeNames.begin(), it));
+    }
+
+    return AircraftPackage; // for backwards-compat
+}
+
 Package::Package(const SGPropertyNode* aProps, CatalogRef aCatalog) :
-    m_catalog(aCatalog)
+    m_catalog(aCatalog.get())
 {
     initWithProps(aProps);
 }
@@ -53,6 +75,16 @@ void Package::initWithProps(const SGPropertyNode* aProps)
     for (auto var : m_props->getChildren("variant")) {
         m_variants.push_back(var->getStringValue("id"));
     }
+
+    if (aProps->hasChild("type")) {
+        m_type = typeFromString(m_props->getStringValue("type"));
+    } else {
+        m_type = AircraftPackage;
+    }
+
+    for (auto p : m_props->getChildren("provides")) {
+        m_provides.push_back(p->getStringValue());
+    }
 }
 
 void Package::updateFromProps(const SGPropertyNode* aProps)
@@ -90,7 +122,7 @@ bool Package::matches(const SGPropertyNode* aFilter) const
 
     if (strutils::starts_with(filter_name, "rating-")) {
         int minRating = aFilter->getIntValue();
-        std::string rname = aFilter->getName() + 7;
+        std::string rname = aFilter->getNameString().c_str() + 7;
         int ourRating = m_props->getChild("rating")->getIntValue(rname, 0);
         return (ourRating >= minRating);
     }
@@ -133,6 +165,12 @@ bool Package::matches(const SGPropertyNode* aFilter) const
         }
     }
 
+    if (filter_name == "type") {
+        handled = true;
+        const auto n = strutils::lowercase(aFilter->getStringValue());
+        return (n == static_typeNames.at(static_cast<int>(m_type)));
+    }
+
     if (!handled) {
       SG_LOG(SG_GENERAL, SG_WARN, "unknown filter term:" << filter_name);
     }
@@ -186,10 +224,26 @@ bool Package::isInstalled() const
     return pathOnDisk().exists();
 }
 
+std::string Package::directoryForType(Type type)
+{
+    switch (type) {
+    case AnyPackageType:
+        throw sg_range_exception("Package::directoryForType: passed 'AnyPackage'");
+    case AircraftPackage:
+    case LibraryPackage:
+    default:
+        return "Aircraft";
+    case AIModelPackage:
+        return "AI/Aircraft";
+    case AddOnPackage:
+        return "Add-On";
+    }
+}
+
 SGPath Package::pathOnDisk() const
 {
     SGPath p(m_catalog->installRoot());
-    p.append("Aircraft");
+    p.append(directoryForType(m_type));
     p.append(dirName());
     return p;
 }
@@ -198,25 +252,63 @@ InstallRef Package::install()
 {
     InstallRef ins = existingInstall();
     if (ins) {
+      // if there's updates, treat this as a 'start update' request
+      if (ins->hasUpdate()) {
+        m_catalog->root()->scheduleToUpdate(ins);
+      }
+
         return ins;
     }
 
   // start a new install
     ins = new Install(this, pathOnDisk());
-    m_catalog->root()->scheduleToUpdate(ins);
+
+
+    if (m_catalog->root()->isOnline()) {
+        m_catalog->root()->scheduleToUpdate(ins);
+    } else {
+        SG_LOG(SG_NETWORK, SG_WARN, "Package::install: skipping since we are offline.");
+    }
 
     _install_cb(this, ins);
 
     return ins;
 }
 
+InstallRef Package::markForInstall()
+{
+    InstallRef ins = existingInstall();
+    if (ins) {
+        return ins;
+    }
+
+    const auto pd = pathOnDisk();
+
+    Dir dir(pd);
+    if (!dir.create(0700)) {
+        SG_LOG(SG_IO, SG_ALERT,
+               "Package::markForInstall: couldn't create directory at:" << pd);
+        return {};
+    }
+
+    ins = new Install{this, pd};
+    _install_cb(this, ins); // not sure if we should trigger the callback for this
+
+    // repeat for dependencies to be kind
+    for (auto dep : dependencies()) {
+        dep->markForInstall();
+    }
+
+    return ins;
+}
+
 InstallRef Package::existingInstall(const InstallCallback& cb) const
 {
     InstallRef install;
     try {
         install = m_catalog->root()->existingInstallForPackage(const_cast<Package*>(this));
     } catch (std::exception& ) {
-        return InstallRef();
+      return {};
     }
 
   if( cb )
@@ -235,6 +327,11 @@ std::string Package::id() const
     return m_id;
 }
 
+CatalogRef Package::catalog() const
+{
+    return {m_catalog};
+}
+
 std::string Package::qualifiedId() const
 {
     return m_catalog->id() + "." + id();
@@ -272,7 +369,7 @@ unsigned int Package::revision() const
 
 std::string Package::name() const
 {
-    return m_props->getStringValue("name");
+    return getLocalisedProp("name", 0);
 }
 
 size_t Package::fileSizeBytes() const
@@ -401,14 +498,8 @@ std::string Package::nameForVariant(const std::string& vid) const
         return name();
     }
 
-    for (auto var : m_props->getChildren("variant")) {
-        if (vid == var->getStringValue("id")) {
-            return var->getStringValue("name");
-        }
-    }
-
-
-    throw sg_exception("Unknow variant +" + vid + " in package " + id());
+    const int index = indexOfVariant(vid);
+    return nameForVariant(index);
 }
 
 unsigned int Package::indexOfVariant(const std::string& vid) const
@@ -426,7 +517,7 @@ unsigned int Package::indexOfVariant(const std::string& vid) const
 
     string_list::const_iterator it = std::find(m_variants.begin(), m_variants.end(), actualId);
     if (it == m_variants.end()) {
-        throw sg_exception("Unknow variant " + vid + " in package " + id());
+        throw sg_exception("Unknown variant " + vid + " in package " + id());
     }
 
     return std::distance(m_variants.begin(), it);
@@ -434,7 +525,7 @@ unsigned int Package::indexOfVariant(const std::string& vid) const
 
 std::string Package::nameForVariant(const unsigned int vIndex) const
 {
-    return propsForVariant(vIndex, "name")->getStringValue("name");
+    return getLocalisedProp("name", vIndex);
 }
 
 SGPropertyNode_ptr Package::propsForVariant(const unsigned int vIndex, const char* propName) const
@@ -469,7 +560,6 @@ std::string Package::parentIdForVariant(unsigned int variantIndex) const
         return m_variants.front();
     }
 
-    assert(indexOfVariant(parentId) >= 0);
     return parentId;
 }
 
@@ -549,6 +639,21 @@ bool Package::validate() const
     return true;
 }
 
+Type Package::type() const
+{
+    return m_type;
+}
+
+string_list Package::providesPaths() const
+{
+    return m_provides;
+}
+
+bool Package::doesProvidePath(const std::string& p) const
+{
+    auto it = std::find(m_provides.begin(), m_provides.end(), p);
+    return it != m_provides.end();
+}
 
 } // of namespace pkg
 
