@@ -85,8 +85,8 @@ private:
   struct State
   {
     State () : node(0), type(""), mode(DEFAULT_MODE), omit(false) {}
-    State (SGPropertyNode * _node, const char * _type, int _mode, bool _omit)
-      : node(_node), type(_type), mode(_mode), omit(_omit) {}
+    State (SGPropertyNode * _node, int _startLine, const char * _type, int _mode, bool _omit)
+      : node(_node), startLine(_startLine), type(_type), mode(_mode), omit(_omit) {}
     bool hasChildren() const
     {
       int n_children = node->nChildren();
@@ -94,6 +94,7 @@ private:
          || (n_children == 1 && node->getChild(0)->getNameString() != ATTR);
     }
     SGPropertyNode * node;
+    int startLine;
     string type;
     int mode;
     bool omit;
@@ -102,11 +103,11 @@ private:
 
   State &state () { return _state_stack[_state_stack.size() - 1]; }
 
-  void push_state (SGPropertyNode * node, const char * type, int mode, bool omit = false) {
+  void push_state (SGPropertyNode * node, int startLine, const char * type, int mode, bool omit = false) {
     if (type == 0)
-      _state_stack.push_back(State(node, "unspecified", mode, omit));
+      _state_stack.push_back(State(node, startLine, "unspecified", mode, omit));
     else
-      _state_stack.push_back(State(node, type, mode, omit));
+      _state_stack.push_back(State(node, startLine, type, mode, omit));
     _level++;
     _data = "";
   }
@@ -161,7 +162,7 @@ setFlag( int& mode,
     string message = "Unrecognized flag value '";
     message += flag;
     message += '\'';
-    throw sg_io_exception(message, location, SG_ORIGIN);
+    throw sg_io_exception(message, location, SG_ORIGIN, false);
   }
 }
 
@@ -176,7 +177,7 @@ PropsVisitor::startElement (const char * name, const XMLAttributes &atts)
       string message = "Root element name is ";
       message += name;
       message += "; expected PropertyList";
-      throw sg_io_exception(message, location, SG_ORIGIN);
+      throw sg_io_exception(message, location, SG_ORIGIN, false);
     }
 
     // Check for an include.
@@ -188,7 +189,7 @@ PropsVisitor::startElement (const char * name, const XMLAttributes &atts)
           {
               string message ="Cannot open file ";
               message += attval;
-              throw sg_io_exception(message, location, SG_ORIGIN);
+              throw sg_io_exception(message, location, SG_ORIGIN, false);
           }
           readProperties(path, _root, 0, _extended);
       } catch (sg_io_exception &e) {
@@ -196,7 +197,7 @@ PropsVisitor::startElement (const char * name, const XMLAttributes &atts)
       }
     }
 
-    push_state(_root, "", DEFAULT_MODE);
+    push_state(_root, getLine(), "", DEFAULT_MODE);
   }
 
   else {
@@ -258,13 +259,11 @@ PropsVisitor::startElement (const char * name, const XMLAttributes &atts)
       // Check for an alias.
       else if( att_name == "alias" )
       {
-        if( !node->alias(val) )
-          SG_LOG
-          (
-            SG_INPUT,
-            SG_ALERT,
-            "Failed to set alias to " << val << "\n at " << location.asString()
-          );
+        if (!node->alias(val, false))
+              SG_LOG(
+                  SG_INPUT,
+                  SG_ALERT,
+                  "Failed to set alias to " << val << "\n at " << location.asString());
       }
 
       // Check for an include.
@@ -278,7 +277,7 @@ PropsVisitor::startElement (const char * name, const XMLAttributes &atts)
           {
             string message ="Cannot open file ";
             message += val;
-            throw sg_io_exception(message, location, SG_ORIGIN);
+            throw sg_io_exception(message, location, SG_ORIGIN, false);
           }
           readProperties(path, node, 0, _extended);
         }
@@ -309,7 +308,7 @@ PropsVisitor::startElement (const char * name, const XMLAttributes &atts)
         attr_node->setUnspecifiedValue(att_name.c_str(), val.c_str());
       }
     }
-    push_state(node, type, mode, omit);
+    push_state(node, getLine(), type, mode, omit);
   }
 }
 
@@ -353,7 +352,7 @@ PropsVisitor::endElement (const char * name)
       string message = "Unrecognized data type '";
       message += st.type;
       message += '\'';
-      throw sg_io_exception(message, location, SG_ORIGIN);
+      throw sg_io_exception(message, location, SG_ORIGIN, false);
     }
     if( !ret )
       SG_LOG
@@ -365,6 +364,9 @@ PropsVisitor::endElement (const char * name)
                          << "\" with type " << st.type
                          << "\n at " << location.asString()
       );
+  } else {
+    // Record the start location of non-leaf nodes
+    st.node->setLocation(SGSourceLocation(getPath(), st.startLine));
   }
 
   // Set the access-mode attributes now,
@@ -377,13 +379,13 @@ PropsVisitor::endElement (const char * name)
     int nChildren = st.node->nChildren();
     for (int i = 0; i < nChildren; i++) {
       SGPropertyNode *src = st.node->getChild(i);
-      const char *name = src->getName();
+      string name = src->getNameString();
       int index = parent.counters[name];
       parent.counters[name]++;
       SGPropertyNode *dst = parent.node->getChild(name, index, true);
       copyProperties(src, dst);
     }
-    parent.node->removeChild(st.node->getName(), st.node->getIndex());
+    parent.node->removeChild(st.node->getNameString(), st.node->getIndex());
   }
   pop_state();
 }
@@ -555,7 +557,7 @@ writeAtts( std::ostream &output,
   if( attr )
     for(int i = 0; i < attr->nChildren(); ++i)
     {
-      output << ' ' << attr->getChild(i)->getName() << "=\"";
+      output << ' ' << attr->getChild(i)->getNameString() << "=\"";
 
       const std::string data = attr->getChild(i)->getStringValue();
       for(int j = 0; j < (int)data.size(); ++j)
@@ -623,7 +625,7 @@ writeNode( std::ostream &output,
   if (!write_all && !isArchivable(node, archive_flag))
     return true; // Everything's OK, but we won't write.
 
-  const string name = node->getName();
+  const string name = node->getNameString();
   int nChildren = node->nChildren();
   const SGPropertyNode* attr_node = node->getChild(ATTR, 0);
   bool attr_written = false,
@@ -689,6 +691,24 @@ writeProperties (ostream &output, const SGPropertyNode * start_node,
   output << "</PropertyList>" << endl;
 }
 
+writePropertiesInline::writePropertiesInline(
+        const SGPropertyNode * start_node,
+        bool write_all,
+        SGPropertyNode::Attribute archive_flag
+        )
+:
+m_start_node(start_node),
+m_write_all(write_all),
+m_archive_flag(archive_flag)
+{
+}
+
+std::ostream& operator<< (std::ostream& out, const writePropertiesInline& wpm)
+{
+    writeProperties(out, wpm.m_start_node, wpm.m_write_all, wpm.m_archive_flag);
+    return out;
+}
+
 
 void
 writeProperties (const SGPath &path, const SGPropertyNode * start_node,
@@ -701,7 +721,7 @@ writeProperties (const SGPath &path, const SGPropertyNode * start_node,
   if (output.good()) {
     writeProperties(output, start_node, write_all, archive_flag);
   } else {
-    throw sg_io_exception("Cannot open file", sg_location(path.utf8Str()));
+    throw sg_io_exception("Cannot open file", sg_location(path.utf8Str()), "", false);
   }
 }
 
@@ -770,7 +790,7 @@ copyPropertyValue(const SGPropertyNode *in, SGPropertyNode *out)
                 break;
             string message = "Unknown internal SGPropertyNode type";
             message += in->getType();
-            throw sg_error(message, SG_ORIGIN);
+            throw sg_error(message, SG_ORIGIN, false);
     }
     
     return retval;
