@@ -411,7 +411,10 @@ SGPickAnimation::apply(osg::Group& group)
   }
 
   group.traverse(*this);
-  
+
+  // Find whether this animation can be harmlessly repeated
+  bool repeatable = isRepeatable();
+
   // iterate over all group children
   int i = group.getNumChildren() - 1;
   for (; 0 <= i; --i) {
@@ -422,7 +425,72 @@ SGPickAnimation::apply(osg::Group& group)
       
     std::list<std::string>::iterator it = std::find(_objectNames.begin(), _objectNames.end(), child->getName());
     if (it != _objectNames.end()) {
-      //_objectNames.erase(it);
+      // Erasing child name from _objectNames should never have been commented
+      // out. Now animations are installed multiple times if the objects they
+      // refer to occur multiple times in the scene graph.
+      if (repeatable) {
+        // Repeating some animations is harmless, so we can stop doing it
+        // without breaking compatibility.
+        _objectNames.erase(it);
+      } else {
+        // Repeating other animations however multiplies their effect.
+        // Detect if we've already installed the animation on this object name,
+        // and warn loudly that this is deprecated behaviour that will change.
+        auto it2 = _objectNamesHandled.find(child->getName());
+        unsigned int* timesHandled;
+        if (it2 != _objectNamesHandled.end())
+          timesHandled = &(*it2).second;
+        else
+          timesHandled = &(_objectNamesHandled[child->getName()] = 0);
+        if (++*timesHandled == 2) {
+          // Second install, Search upwards for a node with multiple parents
+          unsigned int foundMultiParents = 0;
+          unsigned int numParents;
+          for (osg::Node* cur = child; (numParents = cur->getNumParents());
+               cur = cur->getParent(0)) {
+            if (numParents > 1)
+              ++foundMultiParents;
+          }
+          if (!foundMultiParents) {
+            // Its inadvisable to have duplicated object names with animations
+            SG_LOG(SG_GENERAL, SG_DEV_ALERT,
+                   "Warning: " << getType() << " animation applies to multiple distinct objects named \"" << child->getName() << "\".");
+          } else {
+            static bool secondaryWarning = false;
+            if (!secondaryWarning) {
+              SG_LOG(SG_GENERAL, SG_DEV_ALERT, "Deprecation Alert: The knob/slider animations listed below are getting duplicated.");
+              SG_LOG(SG_GENERAL, SG_DEV_ALERT, "    This will be fixed in a future version. Please apply workarounds to ensure compatibility.");
+              SG_LOG(SG_GENERAL, SG_DEV_ALERT, "    For details see: https://wiki.flightgear.org/Knob_and_Slider_Duplication");
+              secondaryWarning = true;
+            }
+
+            SG_LOG(SG_GENERAL, SG_DEV_ALERT, "Warning: " << (1 << foundMultiParents) << "x duplication of " << getType() << " animation on object \"" << child->getName() << "\" is deprecated behaviour (see above)");
+            SG_LOG(SG_GENERAL, SG_DEV_ALERT, "    Animation node at: " << getConfig()->getLocation());
+
+            // Search upwards for a node with multiple parents
+            for (osg::Node* cur = child; (numParents = cur->getNumParents());
+                 cur = cur->getParent(0)) {
+              if (numParents > 1) {
+                // This is the one, look for a parent with a location
+                unsigned int i;
+                for (i = 0; i < numParents; ++i) {
+                  auto* userData = SGSceneUserData::getSceneUserData(cur->getParent(i));
+                  if (userData && userData->getLocation().isValid()) {
+                    SG_LOG(SG_GENERAL, SG_DEV_ALERT, "    Duplicated due to: " << userData->getLocation());
+                    break;
+                  }
+                }
+                if (i == numParents) {
+                  SG_LOG(SG_GENERAL, SG_DEV_ALERT, "    Duplicated scene node '" << cur->getName() << "' at unknown location");
+                  for (i = 0; i < numParents; ++i)
+                    SG_LOG(SG_GENERAL, SG_DEV_ALERT, "        parent " << i << " named '" << cur->getParent(i)->getName() << "'");
+                }
+              }
+            }
+          }
+        }
+      }
+
       install(*child);
       
       osg::ref_ptr<osg::Group> renderGroup, pickGroup;      
@@ -433,12 +501,14 @@ SGPickAnimation::apply(osg::Group& group)
       if (getConfig()->getBoolValue("visible", true)) {
           renderGroup = new osg::Group;
           renderGroup->setName("pick render group");
+          SGSceneUserData::getOrCreateSceneUserData(renderGroup)->setLocation(getConfig()->getLocation());
           renderGroup->addChild(child);
           mainGroup->addChild(renderGroup);
       }
       
       pickGroup = new osg::Group;
       pickGroup->setName("pick highlight group");
+      SGSceneUserData::getOrCreateSceneUserData(pickGroup)->setLocation(getConfig()->getLocation());
       pickGroup->setNodeMask(simgear::PICK_BIT);
       pickGroup->setStateSet(sharedHighlightStateSet());
       mainGroup->addChild(pickGroup);
@@ -506,6 +576,12 @@ SGPickAnimation::setupCallbacks(SGSceneUserData* ud, osg::Group* parent)
   }
 }
 
+bool SGPickAnimation::isRepeatable() const
+{
+    // Pure pick handling doesn't move anything
+    return true;
+}
+
 ///////////////////////////////////////////////////////////////////////////
 
 // insert count copies of binding list A, into the output list.
@@ -827,6 +903,14 @@ SGKnobAnimation::setupCallbacks(SGSceneUserData* ud, osg::Group*)
   ud->setPickCallback(new KnobSliderPickCallback(getConfig(), getModelRoot(), _condition));
 }
 
+bool SGKnobAnimation::isRepeatable() const
+{
+    // For the animation to move anything, there must be an axis and a
+    // non-const-zero animation value.
+    return (!_axis.x() && !_axis.y() && !_axis.z()) ||
+           (_animationValue->isConst() && _animationValue->getValue() == 0);
+}
+
 void SGKnobAnimation::setAlternateMouseWheelDirection(bool aToggle)
 {
     static_knobMouseWheelAlternateDirection = aToggle;
@@ -870,8 +954,8 @@ SGSliderAnimation::SGSliderAnimation(simgear::SGTransientModelData &modelData) :
     SGSharedPtr<SGExpressiond> value = read_value(modelData.getConfigNode(), modelData.getModelRoot(), "-m",
                                                   -SGLimitsd::max(), SGLimitsd::max());
     _animationValue = value->simplify();
-    
-    _axis = readTranslateAxis(modelData.getConfigNode());
+    SGVec3d garbage = SGVec3d::zeros();
+    readRotationCenterAndAxis(modelData.getNode(), garbage, _axis, modelData);
 }
 
 osg::Group*
@@ -893,6 +977,14 @@ SGSliderAnimation::setupCallbacks(SGSceneUserData* ud, osg::Group*)
   ud->setPickCallback(new KnobSliderPickCallback(getConfig(), getModelRoot(), _condition));
 }
 
+bool SGSliderAnimation::isRepeatable() const
+{
+    // For the animation to move anything, there must be an axis and a
+    // non-const-zero animation value.
+    return (!_axis.x() && !_axis.y() && !_axis.z()) ||
+           (_animationValue->isConst() && _animationValue->getValue() == 0);
+}
+
 /*
  * touch screen is a 2d surface that will pass parameters to the callbacks indicating the 
  * normalized coordinates of hover or touch. Touch is defined as a button click. 
