@@ -32,7 +32,7 @@
 
 #include <simgear/compiler.h>
 
-#include <simgear/math/sg_random.h>
+#include <simgear/math/sg_random.hxx>
 #include <simgear/math/sg_geodesy.hxx>
 #include <simgear/scene/util/SGSceneUserData.hxx>
 
