@@ -1,23 +1,6 @@
-/* -*-c++-*-
- *
- * Copyright (C) 2012 Stuart Buchanan
- *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
- * MA 02110-1301, USA.
- *
- */
+// -*-c++-*-
+// SPDX-License-Identifier: GPL-2.0-or-later
+// SPDX-FileCopyrightText: 2012 Stuart Buchanan
 
 #ifdef HAVE_CONFIG_H
 #  include <simgear_config.h>
@@ -45,7 +28,7 @@
 #include <simgear/io/iostreams/sgstream.hxx>
 #include <simgear/math/SGLimits.hxx>
 #include <simgear/math/SGMisc.hxx>
-#include <simgear/math/sg_random.h>
+#include <simgear/math/sg_random.hxx>
 #include <simgear/misc/sg_path.hxx>
 #include <simgear/scene/model/model.hxx>
 #include <simgear/props/props.hxx>
@@ -54,15 +37,15 @@
 #include "SGBuildingBin.hxx"
 
 using namespace osg;
+using std::max;
+using std::min;
 
 namespace simgear
 {
 
-typedef std::map<std::string, osg::observer_ptr<osg::StateSet> > BuildingStateSetMap;
-static BuildingStateSetMap statesetmap;
-
 typedef std::map<std::string, osg::observer_ptr<Effect> > EffectMap;
 static EffectMap buildingEffectMap;
+inline static std::mutex _buildingEffectMap_mutex; // Protects the buildingEffectMap for multi-threaded access
 
 // Helper classes for creating the quad tree
 struct MakeBuildingLeaf
@@ -84,10 +67,10 @@ struct MakeBuildingLeaf
         // vertex shader for texture mapping:
         // (front, roof, roof top vertex, side)
 
-        v->reserve(52);
-        t->reserve(52);
-        n->reserve(52);
-        c->reserve(52);
+        v->reserve(78);
+        t->reserve(78);
+        n->reserve(78);
+        c->reserve(78);
 
         // Now create an OSG Geometry based on the Building
         // 0,0,0 is the bottom center of the front
@@ -99,9 +82,12 @@ struct MakeBuildingLeaf
         v->push_back( osg::Vec3( 0.0, -0.5, -1.0) ); // bottom right
         v->push_back( osg::Vec3( 0.0,  0.5, -1.0) ); // bottom left
         v->push_back( osg::Vec3( 0.0,  0.5,  0.0) ); // top left
-        v->push_back( osg::Vec3( 0.0, -0.5,  0.0) ); // top right
 
-        for (int i=0; i<4; ++i) {
+        v->push_back( osg::Vec3( 0.0,  0.5,  0.0) ); // top left
+        v->push_back( osg::Vec3( 0.0, -0.5,  0.0) ); // top right
+        v->push_back( osg::Vec3( 0.0, -0.5, -1.0) ); // bottom right
+
+        for (int i = 0; i < 6; ++i) {
           n->push_back( osg::Vec3(1, 0, 0) );    // normal
           c->push_back( osg::Vec4(1, 0, 0, 0) ); // color - used to differentiate wall from roof
           t->push_back( osg::Vec2( 0.0, 0.0) );
@@ -111,9 +97,12 @@ struct MakeBuildingLeaf
         v->push_back( osg::Vec3( -1.0, -0.5, -1.0) ); // bottom right
         v->push_back( osg::Vec3(  0.0, -0.5, -1.0) ); // bottom left
         v->push_back( osg::Vec3(  0.0, -0.5,  0.0) ); // top left
-        v->push_back( osg::Vec3( -1.0, -0.5,  0.0) ); // top right
 
-        for (int i=0; i<4; ++i) {
+        v->push_back( osg::Vec3(  0.0, -0.5,  0.0) ); // top left
+        v->push_back( osg::Vec3( -1.0, -0.5,  0.0) ); // top right
+        v->push_back( osg::Vec3( -1.0, -0.5, -1.0) ); // bottom right
+
+        for (int i = 0; i < 6; ++i) {
           n->push_back( osg::Vec3(0, -1, 0) );   // normal
           c->push_back( osg::Vec4(1, 0, 0, 0) ); // color - used to differentiate wall from roof
           t->push_back( osg::Vec2( 0.0, 0.0) );
@@ -123,9 +112,12 @@ struct MakeBuildingLeaf
         v->push_back( osg::Vec3( -1.0,  0.5, -1.0) ); // bottom right
         v->push_back( osg::Vec3( -1.0, -0.5, -1.0) ); // bottom left
         v->push_back( osg::Vec3( -1.0, -0.5,  0.0) ); // top left
-        v->push_back( osg::Vec3( -1.0,  0.5,  0.0) ); // top right
 
-        for (int i=0; i<4; ++i) {
+        v->push_back( osg::Vec3( -1.0, -0.5,  0.0) ); // top left
+        v->push_back( osg::Vec3( -1.0,  0.5,  0.0) ); // top right
+        v->push_back( osg::Vec3( -1.0,  0.5, -1.0) ); // bottom right
+
+        for (int i = 0; i < 6; ++i) {
           n->push_back( osg::Vec3(-1, 0, 0) );   // normal
           c->push_back( osg::Vec4(1, 0, 0, 0) ); // color - used to differentiate wall from roof
           t->push_back( osg::Vec2( 0.0, 0.0) );
@@ -135,9 +127,12 @@ struct MakeBuildingLeaf
         v->push_back( osg::Vec3(  0.0, 0.5, -1.0) ); // bottom right
         v->push_back( osg::Vec3( -1.0, 0.5, -1.0) ); // bottom left
         v->push_back( osg::Vec3( -1.0, 0.5,  0.0) ); // top left
-        v->push_back( osg::Vec3(  0.0, 0.5,  0.0) ); // top right
 
-        for (int i=0; i<4; ++i) {
+        v->push_back( osg::Vec3( -1.0, 0.5,  0.0) ); // top left
+        v->push_back( osg::Vec3(  0.0, 0.5,  0.0) ); // top right
+        v->push_back( osg::Vec3(  0.0, 0.5, -1.0) ); // bottom right
+
+        for (int i = 0; i < 6; ++i) {
           n->push_back( osg::Vec3(0, 1, 0) );    // normal
           c->push_back( osg::Vec4(1, 0, 0, 0) ); // color - used to differentiate wall from roof
           t->push_back( osg::Vec2( 0.0, 0.0) );
@@ -147,15 +142,21 @@ struct MakeBuildingLeaf
         // Front face
         v->push_back( osg::Vec3( 0.0, -0.5, 0.0) ); // bottom right
         v->push_back( osg::Vec3( 0.0,  0.5, 0.0) ); // bottom left
+        v->push_back( osg::Vec3( 0.0,  0.5, 1.0) ); // top left
+
         v->push_back( osg::Vec3( 0.0,  0.5, 1.0) ); // top left
         v->push_back( osg::Vec3( 0.0, -0.5, 1.0) ); // top right
+        v->push_back( osg::Vec3( 0.0, -0.5, 0.0) ); // bottom right
 
         t->push_back( osg::Vec2( 1.0, 0.0) ); // bottom right
         t->push_back( osg::Vec2( 0.0, 0.0) ); // bottom left
         t->push_back( osg::Vec2( 0.0, 1.0) ); // top left
-        t->push_back( osg::Vec2( 1.0, 1.0) ); // top right
 
-        for (int i=0; i<4; ++i) {
+        t->push_back( osg::Vec2( 0.0, 1.0) ); // top left
+        t->push_back( osg::Vec2( 1.0, 1.0) ); // top right
+        t->push_back( osg::Vec2( 1.0, 0.0) ); // bottom right
+
+        for (int i = 0; i < 6; ++i) {
           n->push_back( osg::Vec3(1, 0, 0) );    // normal
           c->push_back( osg::Vec4(1, 0, 0, 0) ); // color - used to differentiate wall from roof
         }
@@ -163,15 +164,21 @@ struct MakeBuildingLeaf
         // Left face
         v->push_back( osg::Vec3( -1.0, -0.5, 0.0) ); // bottom right
         v->push_back( osg::Vec3(  0.0, -0.5, 0.0) ); // bottom left
+        v->push_back( osg::Vec3(  0.0, -0.5, 1.0) ); // top left
+
         v->push_back( osg::Vec3(  0.0, -0.5, 1.0) ); // top left
         v->push_back( osg::Vec3( -1.0, -0.5, 1.0) ); // top right
+        v->push_back( osg::Vec3( -1.0, -0.5, 0.0) ); // bottom right
 
         t->push_back( osg::Vec2( 0.0, 0.0) ); // bottom right
         t->push_back( osg::Vec2( 1.0, 0.0) ); // bottom left
         t->push_back( osg::Vec2( 1.0, 1.0) ); // top left
-        t->push_back( osg::Vec2( 0.0, 1.0) ); // top right
 
-        for (int i=0; i<4; ++i) {
+        t->push_back( osg::Vec2( 1.0, 1.0) ); // top left
+        t->push_back( osg::Vec2( 0.0, 1.0) ); // top right
+        t->push_back( osg::Vec2( 0.0, 0.0) ); // bottom right
+
+        for (int i = 0; i < 6; ++i) {
           n->push_back( osg::Vec3(0, -1, 0) );    // normal
           c->push_back( osg::Vec4(0, 0, 0, 1) ); // color - used to differentiate wall from roof
         }
@@ -179,15 +186,21 @@ struct MakeBuildingLeaf
         // Back face
         v->push_back( osg::Vec3( -1.0,  0.5, 0.0) ); // bottom right
         v->push_back( osg::Vec3( -1.0, -0.5, 0.0) ); // bottom left
+        v->push_back( osg::Vec3( -1.0, -0.5, 1.0) ); // top left
+
         v->push_back( osg::Vec3( -1.0, -0.5, 1.0) ); // top left
         v->push_back( osg::Vec3( -1.0,  0.5, 1.0) ); // top right
+        v->push_back( osg::Vec3( -1.0,  0.5, 0.0) ); // bottom right
 
         t->push_back( osg::Vec2( 1.0, 0.0) ); // bottom right
         t->push_back( osg::Vec2( 0.0, 0.0) ); // bottom left
         t->push_back( osg::Vec2( 0.0, 1.0 ) ); // top left
-        t->push_back( osg::Vec2( 1.0, 1.0 ) ); // top right
 
-        for (int i=0; i<4; ++i) {
+        t->push_back( osg::Vec2( 0.0, 1.0 ) ); // top left
+        t->push_back( osg::Vec2( 1.0, 1.0 ) ); // top right
+        t->push_back( osg::Vec2( 1.0, 0.0) ); // bottom right
+
+        for (int i = 0; i < 6; ++i) {
           n->push_back( osg::Vec3(-1, 0, 0) );    // normal
           c->push_back( osg::Vec4(1, 0, 0, 0) ); // color - used to differentiate wall from roof
         }
@@ -195,15 +208,21 @@ struct MakeBuildingLeaf
         // Right face
         v->push_back( osg::Vec3(  0.0, 0.5, 0.0) ); // bottom right
         v->push_back( osg::Vec3( -1.0, 0.5, 0.0) ); // bottom left
+        v->push_back( osg::Vec3( -1.0, 0.5, 1.0) ); // top left
+
         v->push_back( osg::Vec3( -1.0, 0.5, 1.0) ); // top left
         v->push_back( osg::Vec3(  0.0, 0.5, 1.0) ); // top right
+        v->push_back( osg::Vec3(  0.0, 0.5, 0.0) ); // bottom right
 
         t->push_back( osg::Vec2( 0.0, 0.0) ); // bottom right
         t->push_back( osg::Vec2( 1.0, 0.0) ); // bottom left
         t->push_back( osg::Vec2( 1.0, 1.0 ) ); // top left
-        t->push_back( osg::Vec2( 0.0, 1.0 ) ); // top right
 
-        for (int i=0; i<4; ++i) {
+        t->push_back( osg::Vec2( 1.0, 1.0 ) ); // top left
+        t->push_back( osg::Vec2( 0.0, 1.0 ) ); // top right
+        t->push_back( osg::Vec2( 0.0, 0.0) ); // bottom right
+
+        for (int i = 0; i < 6; ++i) {
           n->push_back( osg::Vec3(0, 1, 0) );    // normal
           c->push_back( osg::Vec4(0, 0, 0, 1) ); // color - used to differentiate wall from roof
         }
@@ -212,107 +231,152 @@ struct MakeBuildingLeaf
         // Front face
         v->push_back( osg::Vec3( 0.0, -0.5, 1.0) ); // bottom right
         v->push_back( osg::Vec3( 0.0,  0.5, 1.0) ); // bottom left
+        v->push_back( osg::Vec3( 0.0,  0.5, 1.0) ); // top left
+
         v->push_back( osg::Vec3( 0.0,  0.5, 1.0) ); // top left
         v->push_back( osg::Vec3( 0.0, -0.5, 1.0) ); // top right
+        v->push_back( osg::Vec3( 0.0, -0.5, 1.0) ); // bottom right
 
         t->push_back( osg::Vec2( -1.0, 0.0) ); // bottom right
         t->push_back( osg::Vec2(  0.0, 0.0) ); // bottom left
+        t->push_back( osg::Vec2(  0.0, 1.0) ); // top left
+
         t->push_back( osg::Vec2(  0.0, 1.0) ); // top left
         t->push_back( osg::Vec2( -1.0, 1.0) ); // top right
+        t->push_back( osg::Vec2( -1.0, 0.0) ); // bottom right
 
         c->push_back( osg::Vec4(0, 1, 0, 0) ); // color - used to differentiate wall from roof
         c->push_back( osg::Vec4(0, 1, 0, 0) ); // color - used to differentiate wall from roof
         c->push_back( osg::Vec4(0, 1, 1, 0) ); // color - used to differentiate wall from roof
-        c->push_back( osg::Vec4(0, 1, 1, 0) ); // color - used to differentiate wall from roof
 
-        for (int i=0; i<4; ++i) {
+        c->push_back( osg::Vec4(0, 1, 1, 0) ); // color - used to differentiate wall from roof
+        c->push_back( osg::Vec4(0, 1, 1, 0) ); // color - used to differentiate wall from roof
+        c->push_back( osg::Vec4(0, 1, 0, 0) ); // color - used to differentiate wall from roof
+
+        for (int i = 0; i < 6; ++i) {
           n->push_back( osg::Vec3(0.707, 0, 0.707) );    // normal
         }
 
         // Left face
         v->push_back( osg::Vec3( -1.0, -0.5, 1.0) ); // bottom right
         v->push_back( osg::Vec3(  0.0, -0.5, 1.0) ); // bottom left
+        v->push_back( osg::Vec3(  0.0, -0.5, 1.0) ); // top left
+
         v->push_back( osg::Vec3(  0.0, -0.5, 1.0) ); // top left
         v->push_back( osg::Vec3( -1.0, -0.5, 1.0) ); // top right
+        v->push_back( osg::Vec3( -1.0, -0.5, 1.0) ); // bottom right
 
         t->push_back( osg::Vec2(  0.0, 0.0) ); // bottom right
         t->push_back( osg::Vec2( -1.0, 0.0) ); // bottom left
+        t->push_back( osg::Vec2( -1.0, 1.0) ); // top left
+
         t->push_back( osg::Vec2( -1.0, 1.0) ); // top left
         t->push_back( osg::Vec2(  0.0, 1.0) ); // top right
+        t->push_back( osg::Vec2(  0.0, 0.0) ); // bottom right
 
         c->push_back( osg::Vec4(0, 1, 0, 0) ); // color - used to differentiate wall from roof
         c->push_back( osg::Vec4(0, 1, 0, 0) ); // color - used to differentiate wall from roof
         c->push_back( osg::Vec4(0, 1, 1, 0) ); // color - used to differentiate wall from roof
-        c->push_back( osg::Vec4(0, 1, 1, 0) ); // color - used to differentiate wall from roof
 
-        for (int i=0; i<4; ++i) {
+        c->push_back( osg::Vec4(0, 1, 1, 0) ); // color - used to differentiate wall from roof
+        c->push_back( osg::Vec4(0, 1, 1, 0) ); // color - used to differentiate wall from roof
+        c->push_back( osg::Vec4(0, 1, 0, 0) ); // color - used to differentiate wall from roof
+
+        for (int i = 0; i < 6; ++i) {
           n->push_back( osg::Vec3(0, -0.707, 0.707) );    // normal
         }
 
         // Back face
         v->push_back( osg::Vec3( -1.0,  0.5, 1.0) ); // bottom right
         v->push_back( osg::Vec3( -1.0, -0.5, 1.0) ); // bottom left
+        v->push_back( osg::Vec3( -1.0, -0.5, 1.0) ); // top left
+
         v->push_back( osg::Vec3( -1.0, -0.5, 1.0) ); // top left
         v->push_back( osg::Vec3( -1.0,  0.5, 1.0) ); // top right
+        v->push_back( osg::Vec3( -1.0,  0.5, 1.0) ); // bottom right
 
         t->push_back( osg::Vec2( -1.0, 0.0) ); // bottom right
         t->push_back( osg::Vec2(  0.0, 0.0) ); // bottom left
+        t->push_back( osg::Vec2(  0.0, 1.0 ) ); // top left
+
         t->push_back( osg::Vec2(  0.0, 1.0 ) ); // top left
         t->push_back( osg::Vec2( -1.0, 1.0 ) ); // top right
+        t->push_back( osg::Vec2( -1.0, 0.0) ); // bottom right
 
         c->push_back( osg::Vec4(0, 1, 0, 0) ); // color - used to differentiate wall from roof
         c->push_back( osg::Vec4(0, 1, 0, 0) ); // color - used to differentiate wall from roof
         c->push_back( osg::Vec4(0, 1, 1, 0) ); // color - used to differentiate wall from roof
-        c->push_back( osg::Vec4(0, 1, 1, 0) ); // color - used to differentiate wall from roof
 
-        for (int i=0; i<4; ++i) {
+        c->push_back( osg::Vec4(0, 1, 1, 0) ); // color - used to differentiate wall from roof
+        c->push_back( osg::Vec4(0, 1, 1, 0) ); // color - used to differentiate wall from roof
+        c->push_back( osg::Vec4(0, 1, 0, 0) ); // color - used to differentiate wall from roof
+
+        for (int i = 0; i < 6; ++i) {
           n->push_back( osg::Vec3(-0.707, 0, 0.707) );    // normal
         }
 
         // Right face
         v->push_back( osg::Vec3(  0.0, 0.5, 1.0) ); // bottom right
         v->push_back( osg::Vec3( -1.0, 0.5, 1.0) ); // bottom left
+        v->push_back( osg::Vec3( -1.0, 0.5, 1.0) ); // top left
+
         v->push_back( osg::Vec3( -1.0, 0.5, 1.0) ); // top left
         v->push_back( osg::Vec3(  0.0, 0.5, 1.0) ); // top right
+        v->push_back( osg::Vec3(  0.0, 0.5, 1.0) ); // bottom right
 
         t->push_back( osg::Vec2(  0.0, 0.0) ); // bottom right
         t->push_back( osg::Vec2( -1.0, 0.0) ); // bottom left
+        t->push_back( osg::Vec2( -1.0, 1.0 ) ); // top left
+
         t->push_back( osg::Vec2( -1.0, 1.0 ) ); // top left
         t->push_back( osg::Vec2(  0.0, 1.0 ) ); // top right
+        t->push_back( osg::Vec2(  0.0, 0.0) ); // bottom right
 
         c->push_back( osg::Vec4(0, 1, 0, 0) ); // color - used to differentiate wall from roof
         c->push_back( osg::Vec4(0, 1, 0, 0) ); // color - used to differentiate wall from roof
         c->push_back( osg::Vec4(0, 1, 1, 0) ); // color - used to differentiate wall from roof
-        c->push_back( osg::Vec4(0, 1, 1, 0) ); // color - used to differentiate wall from roof
 
-        for (int i=0; i<4; ++i) {
+        c->push_back( osg::Vec4(0, 1, 1, 0) ); // color - used to differentiate wall from roof
+        c->push_back( osg::Vec4(0, 1, 1, 0) ); // color - used to differentiate wall from roof
+        c->push_back( osg::Vec4(0, 1, 0, 0) ); // color - used to differentiate wall from roof
+
+        for (int i = 0; i < 6; ++i) {
           n->push_back( osg::Vec3(0, 0.707, 0.707) );    // normal
         }
 
         // Top face
         v->push_back( osg::Vec3(  0.0, -0.5, 1.0) ); // bottom right
         v->push_back( osg::Vec3(  0.0,  0.5, 1.0) ); // bottom left
+        v->push_back( osg::Vec3( -1.0,  0.5, 1.0) ); // top left
+
         v->push_back( osg::Vec3( -1.0,  0.5, 1.0) ); // top left
         v->push_back( osg::Vec3( -1.0, -0.5, 1.0) ); // top right
+        v->push_back( osg::Vec3(  0.0, -0.5, 1.0) ); // bottom right
 
         t->push_back( osg::Vec2( -1.0, 0.0) ); // bottom right
         t->push_back( osg::Vec2(  0.0, 0.0) ); // bottom left
+        t->push_back( osg::Vec2(  0.0, 1.0) ); // top left
+
         t->push_back( osg::Vec2(  0.0, 1.0) ); // top left
         t->push_back( osg::Vec2( -1.0, 1.0) ); // top right
+        t->push_back( osg::Vec2( -1.0, 0.0) ); // bottom right
 
         c->push_back( osg::Vec4(0, 1, 1, 0) ); // color - used to differentiate wall from roof
+        c->push_back( osg::Vec4(0, 1, 1, 0) ); // color - used to differentiate wall from roof
+        c->push_back( osg::Vec4(0, 1, 1, 0) ); // color - used to differentiate wall from roof
+
         c->push_back( osg::Vec4(0, 1, 1, 0) ); // color - used to differentiate wall from roof
         c->push_back( osg::Vec4(0, 1, 1, 0) ); // color - used to differentiate wall from roof
         c->push_back( osg::Vec4(0, 1, 1, 0) ); // color - used to differentiate wall from roof
 
-        for (int i=0; i<4; ++i) {
-          n->push_back( osg::Vec3(0, 0, -1.0) );    // normal
+        for (int i = 0; i < 6; ++i) {
+          n->push_back( osg::Vec3(0, 0, 1.0) );    // normal
         }
 
-        assert(v->size() == 52);
-        assert(t->size() == 52);
-        assert(c->size() == 52);
-        assert(n->size() == 52);
+        assert(v->size() == 78);
+        assert(t->size() == 78);
+        assert(c->size() == 78);
+        assert(n->size() == 78);
 
         Geometry* geom = new Geometry;
         static int buildingCounter = 0;
@@ -330,8 +394,7 @@ struct MakeBuildingLeaf
         geom->setVertexAttribArray(BUILDING_ATTR1, new osg::Vec3Array, Array::BIND_PER_VERTEX);
         geom->setVertexAttribArray(BUILDING_ATTR2, new osg::Vec3Array, Array::BIND_PER_VERTEX);
 
-        //geom->addPrimitiveSet( new osg::DrawArrays( GL_QUADS, 0, 48, 0) );
-        geom->addPrimitiveSet( new osg::DrawArrays( GL_QUADS, 0, 52, 0) );
+        geom->addPrimitiveSet(new osg::DrawArrays(GL_TRIANGLES, 0, 78, 0));
 
         EffectGeode* geode = new EffectGeode;
         geode->addDrawable(geom);
@@ -419,11 +482,6 @@ typedef QuadTreeBuilder<LOD*, SGBuildingBin::BuildingInstance, MakeBuildingLeaf,
   SGBuildingBin::SGBuildingBin(const SGPath& absoluteFileName, const SGMaterial *mat, bool useVBOs) :
     SGBuildingBin::SGBuildingBin(mat, useVBOs)
   {
-    if (!absoluteFileName.exists()) {
-      SG_LOG(SG_TERRAIN, SG_ALERT, "Building list file " << absoluteFileName << " does not exist.");
-      return;
-    }
-
     sg_gzifstream stream(absoluteFileName);
     if (!stream.is_open()) {
       SG_LOG(SG_TERRAIN, SG_ALERT, "Unable to open " << absoluteFileName);
@@ -484,8 +542,10 @@ typedef QuadTreeBuilder<LOD*, SGBuildingBin::BuildingInstance, MakeBuildingLeaf,
       if (b == 1)  type = BuildingType::MEDIUM;
       if (b == 2)  type = BuildingType::LARGE;
 
-      // Rotation is in the file as degrees, but in the datastructure normalized
-      // to 0.0 - 1.0
+      // Rotation is in the file as degrees
+      // First we wrap the rotation value to be between 0 and 360, even if it's negative
+      r = SGMisc<float>::normalizePeriodic(0.0f, 360.f, r);
+      // In the datastructure, rotation needs to be normalized to 0.0 - 1.0
       float rot = (float) (r / 360.0f);
 
       if (w == 0.0f) {
@@ -512,7 +572,9 @@ typedef QuadTreeBuilder<LOD*, SGBuildingBin::BuildingInstance, MakeBuildingLeaf,
       }
 
       _textureName = mat->get_building_texture();
+      _normalMapName = mat->get_building_normalmap();
       _lightMapName = mat->get_building_lightmap();
+      _reflectionMapName = mat->get_building_reflectionmap();
       buildingRange = mat->get_building_range();
       SG_LOG(SG_TERRAIN, SG_DEBUG, "Building texture " << _textureName);
   }
@@ -634,9 +696,10 @@ typedef QuadTreeBuilder<LOD*, SGBuildingBin::BuildingInstance, MakeBuildingLeaf,
       if ((roof_shape == 3) || (roof_shape == 4) || (roof_shape == 7)) {
         // Hipped, half-hipped, mansard
         // The pitch height expressed as a fraction of the building width/depth such that the hipped
-        // roof has a pitch of around 45 degrees.  A minimum of 0.5 so that they have at least some ridge.
-        if (roof_orientation == 0) rooftop_scale = Vec2f(0.0f, max(0.5f,(depth -  2*pitch_height) / width));
-        if (roof_orientation == 1) rooftop_scale = Vec2f(max(0.5f,(width -  2*pitch_height) / width), 0.0f);
+        // roof has a pitch of around 45 degrees. A minimum of 0.5 so that they have at least some ridge
+        // and a maximum of 1.0 to maintain a reasonable roof shape.
+        if (roof_orientation == 0) rooftop_scale = Vec2f(0.0f, std::clamp((depth -  2*pitch_height) / width, 0.5f, 1.0f));
+        if (roof_orientation == 1) rooftop_scale = Vec2f(std::clamp((width -  2*pitch_height) / width, 0.5f, 1.0f), 0.0f);
       }
 
       // Pyramidal, dome, onion
@@ -656,69 +719,68 @@ typedef QuadTreeBuilder<LOD*, SGBuildingBin::BuildingInstance, MakeBuildingLeaf,
     int roof_orientation;
 
     // Generate a random seed for the building generation.
-    mt seed;
-    mt_init(&seed, unsigned(p.x() + p.y() + p.z()));
+    pc_init(unsigned(p.x() + p.y() + p.z()));
 
     if (buildingtype == SGBuildingBin::SMALL) {
       // Small building
       // Maximum number of floors is 3, and maximum width/depth is 192m.
-      width = _material->get_building_small_min_width() + mt_rand(&seed) * mt_rand(&seed) * (_material->get_building_small_max_width() - _material->get_building_small_min_width());
-      depth = _material->get_building_small_min_depth() + mt_rand(&seed) * mt_rand(&seed) * (_material->get_building_small_max_depth() - _material->get_building_small_min_depth());
-      floors = SGMisc<double>::round(_material->get_building_small_min_floors() + mt_rand(&seed) * (_material->get_building_small_max_floors() - _material->get_building_small_min_floors()));
-      height = floors * (2.8 + mt_rand(&seed));
+      width = _material->get_building_small_min_width() + pc_rand() * pc_rand() * (_material->get_building_small_max_width() - _material->get_building_small_min_width());
+      depth = _material->get_building_small_min_depth() + pc_rand() * pc_rand() * (_material->get_building_small_max_depth() - _material->get_building_small_min_depth());
+      floors = SGMisc<double>::round(_material->get_building_small_min_floors() + pc_rand() * (_material->get_building_small_max_floors() - _material->get_building_small_min_floors()));
+      height = floors * (2.8 + pc_rand());
 
       // Small buildings are never deeper than they are wide.
       if (depth > width) { depth = width; }
 
-      pitch_height = (mt_rand(&seed) < _material->get_building_small_pitch()) ? 3.0 : 0.0;
+      pitch_height = (pc_rand() < _material->get_building_small_pitch()) ? 3.0 : 0.0;
 
       if (pitch_height == 0.0) {
         roof_shape = 0;
         roof_orientation = 0;
       } else {
-        roof_shape = (int) (mt_rand(&seed) * 10.0);
-        roof_orientation = (int) std::round((float) mt_rand(&seed));
+        roof_shape = (int) (pc_rand() * 10.0);
+        roof_orientation = (int) std::round((float) pc_rand());
       }
     } else if (buildingtype == SGBuildingBin::MEDIUM) {
       // MEDIUM BUILDING
-      width = _material->get_building_medium_min_width() + mt_rand(&seed) * mt_rand(&seed) * (_material->get_building_medium_max_width() - _material->get_building_medium_min_width());
-      depth = _material->get_building_medium_min_depth() + mt_rand(&seed) * mt_rand(&seed) * (_material->get_building_medium_max_depth() - _material->get_building_medium_min_depth());
-      floors = SGMisc<double>::round(_material->get_building_medium_min_floors() + mt_rand(&seed) * (_material->get_building_medium_max_floors() - _material->get_building_medium_min_floors()));
-      height = floors * (2.8 + mt_rand(&seed));
+      width = _material->get_building_medium_min_width() + pc_rand() * pc_rand() * (_material->get_building_medium_max_width() - _material->get_building_medium_min_width());
+      depth = _material->get_building_medium_min_depth() + pc_rand() * pc_rand() * (_material->get_building_medium_max_depth() - _material->get_building_medium_min_depth());
+      floors = SGMisc<double>::round(_material->get_building_medium_min_floors() + pc_rand() * (_material->get_building_medium_max_floors() - _material->get_building_medium_min_floors()));
+      height = floors * (2.8 + pc_rand());
 
       while ((height > width) && (floors > _material->get_building_medium_min_floors())) {
           // Ensure that medium buildings aren't taller than they are wide
           floors--;
-          height = floors * (2.8 + mt_rand(&seed));
+          height = floors * (2.8 + pc_rand());
       }
 
-      pitch_height = (mt_rand(&seed) < _material->get_building_medium_pitch()) ? 3.0 : 0.0;
+      pitch_height = (pc_rand() < _material->get_building_medium_pitch()) ? 3.0 : 0.0;
 
       if (pitch_height == 0.0) {
         roof_shape = 0;
         roof_orientation = 0;
       } else {
-        roof_shape = (int) (mt_rand(&seed) * 10.0);
-        roof_orientation = (int) std::round((float) mt_rand(&seed));
+        roof_shape = (int) (pc_rand() * 10.0);
+        roof_orientation = (int) std::round((float) pc_rand());
       }
     } else {
       // LARGE BUILDING
-      width = _material->get_building_large_min_width() + mt_rand(&seed) * (_material->get_building_large_max_width() - _material->get_building_large_min_width());
-      depth = _material->get_building_large_min_depth() + mt_rand(&seed) * (_material->get_building_large_max_depth() - _material->get_building_large_min_depth());
-      floors = SGMisc<double>::round(_material->get_building_large_min_floors() + mt_rand(&seed) * (_material->get_building_large_max_floors() - _material->get_building_large_min_floors()));
-      height = floors * (2.8 + mt_rand(&seed));
-      pitch_height = (mt_rand(&seed) < _material->get_building_large_pitch()) ? 3.0 : 0.0;
+      width = _material->get_building_large_min_width() + pc_rand() * (_material->get_building_large_max_width() - _material->get_building_large_min_width());
+      depth = _material->get_building_large_min_depth() + pc_rand() * (_material->get_building_large_max_depth() - _material->get_building_large_min_depth());
+      floors = SGMisc<double>::round(_material->get_building_large_min_floors() + pc_rand() * (_material->get_building_large_max_floors() - _material->get_building_large_min_floors()));
+      height = floors * (2.8 + pc_rand());
+      pitch_height = (pc_rand() < _material->get_building_large_pitch()) ? 3.0 : 0.0;
 
       if (pitch_height == 0.0) {
         roof_shape = 0;
         roof_orientation = 0;
       } else {
-        roof_shape = (int) (mt_rand(&seed) * 10.0);
-        roof_orientation = (int) std::round((float) mt_rand(&seed));
+        roof_shape = (int) (pc_rand() * 10.0);
+        roof_orientation = (int) std::round((float) pc_rand());
       }
     }
 
-    insert(p, r, buildingtype, width, depth, height, pitch_height, floors, roof_shape, roof_orientation, (int) (mt_rand(&seed) * 1000.0), (int) (mt_rand(&seed) * 1000.0));
+    insert(p, r, buildingtype, width, depth, height, pitch_height, floors, roof_shape, roof_orientation, (int) (pc_rand() * 1000.0), (int) (pc_rand() * 1000.0));
   }
 
   int SGBuildingBin::getNumBuildings() {
@@ -764,25 +826,35 @@ typedef QuadTreeBuilder<LOD*, SGBuildingBin::BuildingInstance, MakeBuildingLeaf,
   ref_ptr<Group> SGBuildingBin::createBuildingsGroup(Matrix transInv, const SGReaderWriterOptions* options)
   {
     ref_ptr<Effect> effect;
-    auto iter = buildingEffectMap.find(_textureName);
 
-    if ((iter == buildingEffectMap.end())||
-        (!iter->second.lock(effect)))
     {
-      SGPropertyNode_ptr effectProp = new SGPropertyNode;
-      makeChild(effectProp, "inherits-from")->setStringValue("Effects/building");
-      SGPropertyNode* params = makeChild(effectProp, "parameters");
-      // Main texture - n=0
-      params->getChild("texture", 0, true)->getChild("image", 0, true)->setStringValue(_textureName);
+      const std::lock_guard<std::mutex> lock(_buildingEffectMap_mutex); // Lock the buildingEffectMap for this scope
+      auto iter = buildingEffectMap.find(_textureName);
 
-      // Light map - n=3
-      params->getChild("texture", 3, true)->getChild("image", 0, true)->setStringValue(_lightMapName);
+      if ((iter == buildingEffectMap.end())||
+          (!iter->second.lock(effect)))
+      {
+        SGPropertyNode_ptr effectProp = new SGPropertyNode;
+        makeChild(effectProp, "inherits-from")->setStringValue("Effects/building");
+        SGPropertyNode* params = makeChild(effectProp, "parameters");
+        // Main texture - n=0
+        params->getChild("texture", 0, true)->getChild("image", 0, true)->setStringValue(_textureName);
 
-      effect = makeEffect(effectProp, true, options);
-      if (iter == buildingEffectMap.end())
-          buildingEffectMap.insert(EffectMap::value_type(_textureName, effect));
-      else
-          iter->second = effect; // update existing, but empty observer
+        // Normal map - n=2
+        params->getChild("texture", 2, true)->getChild("image", 0, true)->setStringValue(_normalMapName);
+
+        // Light map - n=3
+        params->getChild("texture", 3, true)->getChild("image", 0, true)->setStringValue(_lightMapName);
+
+        // Reflection map - n=4
+        params->getChild("texture", 4, true)->getChild("image", 0, true)->setStringValue(_reflectionMapName);
+
+        effect = makeEffect(effectProp, true, options);
+        if (iter == buildingEffectMap.end())
+            buildingEffectMap.insert(EffectMap::value_type(_textureName, effect));
+        else
+            iter->second = effect; // update existing, but empty observer
+      }
     }
 
     // Transform building positions from the "geocentric" positions we
@@ -863,7 +935,7 @@ typedef QuadTreeBuilder<LOD*, SGBuildingBin::BuildingInstance, MakeBuildingLeaf,
       SGBuildingBinList::iterator i;
 
       for (i = buildings.begin(); i != buildings.end(); ++i) {
-          SGBuildingBin* bin = *i;
+          SGBuildingBinRef bin = *i;
           ref_ptr<Group> group = bin->createBuildingsGroup(transInv, options);
 
           for (size_t j = 0; j < group->getNumChildren(); ++j) {
