@@ -64,6 +64,10 @@ using std::vector;
 #ifndef ALC_ALL_DEVICES_SPECIFIER
 # define ALC_ALL_DEVICES_SPECIFIER	0x1013
 #endif
+#ifndef AL_EXT_SOURCE_RADIUS
+#define AL_EXT_SOURCE_RADIUS 1
+#define AL_SOURCE_RADIUS		0x1031
+#endif
 #ifndef AL_FORMAT_MONO_MULAW_EXT
 # define AL_FORMAT_MONO_MULAW_EXT	0x10014
 #endif
@@ -102,6 +106,7 @@ public:
     {
         _at_up_vec[0] = 0.0; _at_up_vec[1] = 0.0; _at_up_vec[2] = -1.0;
         _at_up_vec[3] = 0.0; _at_up_vec[4] = 1.0; _at_up_vec[5] = 0.0;
+        _source_radius = alIsExtensionPresent((ALchar *)"AL_EXT_SOURCE_RADIUS");
     }
     
     void update_pos_and_orientation()
@@ -143,6 +148,7 @@ public:
     ALfloat _at_up_vec[6];
 
     bool _bad_doppler = false;
+    bool _source_radius = false;
     
     sample_group_map _sample_groups;
     buffer_map _buffers;
@@ -157,8 +163,6 @@ public:
 SGSoundMgr::SGSoundMgr() {
     d.reset(new SoundManagerPrivate);
     d->_base_pos = SGVec3d::fromGeod(_geod_pos);
-
-    _block_support = alIsExtensionPresent((ALchar *)"AL_SOFT_block_alignment");
 }
 
 // destructor
@@ -198,12 +202,14 @@ void SGSoundMgr::init()
     }
 
     if ((!devname)||(testForError(device, "Audio device not available, trying default.")) ) {
+        // REVIEW: Memory Leak - 26 bytes in 1 blocks are still reachable
         device = alcOpenDevice(nullptr);
         if (testForError(device, "Default audio device not available.") ) {
            return;
         }
     }
 
+    // REVIEW: Memory Leak - 64 bytes in 1 blocks are still reachable
     ALCcontext *context = alcCreateContext(device, nullptr);
     testForALCError("context creation.");
     if ( testForError(context, "Unable to create a valid context.") ) {
@@ -235,6 +241,8 @@ void SGSoundMgr::init()
     alDopplerFactor(1.0f);
     alDopplerVelocity(_sound_velocity);
 
+    _block_support = alIsExtensionPresent((ALchar *)"AL_SOFT_block_alignment");
+
     // gain = AL_REFERENCE_DISTANCE / (AL_REFERENCE_DISTANCE +
     //        AL_ROLLOFF_FACTOR * (distance - AL_REFERENCE_DISTANCE));
     alDistanceModel(AL_INVERSE_DISTANCE_CLAMPED);
@@ -426,6 +434,7 @@ bool SGSoundMgr::add( SGSampleGroup *sgrp, const std::string& refname )
     auto sample_grp_it = d->_sample_groups.find( refname );
     if ( sample_grp_it != d->_sample_groups.end() ) {
         // sample group already exists
+        SG_LOG(SG_SOUND, SG_DEV_ALERT, "Duplicate add of sample-group:" << refname);
         return false;
     }
 
@@ -465,8 +474,8 @@ SGSampleGroup *SGSoundMgr::find( const std::string &refname, bool create ) {
     if ( sample_grp_it == d->_sample_groups.end() ) {
         // sample group was not found.
         if (create) {
+            // creating a group implicitly calls add()
             SGSampleGroup* sgrp = new SGSampleGroup(this, refname);
-            add( sgrp, refname );
             return sgrp;
         }
         else 
@@ -594,6 +603,7 @@ unsigned int SGSoundMgr::request_buffer(SGSoundSample *sample)
         case SG_SAMPLE_STEREO8:
             SG_LOG(SG_SOUND, SG_POPUP, "Stereo sound detected:\n" << sample->get_sample_name() << "\nUse two separate mono files instead if required.");
             format = AL_FORMAT_STEREO8;
+            break;
         default:
             SG_LOG(SG_SOUND, SG_ALERT, "unsupported audio format");
             return buffer;
@@ -789,6 +799,10 @@ void SGSoundMgr::update_sample_config( SGSoundSample *sample, SGVec3d& position,
         alSourcef( source, AL_MAX_DISTANCE, sample->get_max_dist() );
         alSourcef( source, AL_REFERENCE_DISTANCE,
                            sample->get_reference_dist() );
+        if (d->_source_radius) {
+            alSourcef( source, AL_SOURCE_RADIUS,
+                               0.5f*sample->get_reference_dist() );
+        }
         testForError("distance rolloff");
     }
 }
@@ -812,13 +826,14 @@ bool SGSoundMgr::load( const std::string &samplepath,
 
     auto data = simgear::loadWAVFromFile(samplepath, format, size, freqf, blocksz);
     freq = (ALsizei)freqf;
-    if (data == nullptr) {
-        throw sg_io_exception("Failed to load wav file", sg_location(samplepath));
+    if (!data) {
+        return false;
     }
 
     if (format == AL_FORMAT_STEREO8 || format == AL_FORMAT_STEREO16) {
          free(data);
-        throw sg_io_exception("Warning: STEREO files are not supported for 3D audio effects: " + samplepath);
+        SG_LOG(SG_IO, SG_DEV_ALERT, "Warning: STEREO files are not supported for 3D audio effects: " << samplepath);
+        return false;
     }
 
     *dbuf = (void *)data;
@@ -837,6 +852,7 @@ vector<std::string> SGSoundMgr::get_available_devices()
     const ALCchar *s;
 
     if (alcIsExtensionPresent(nullptr, "ALC_enumerate_all_EXT") == AL_TRUE) {
+        // REVIEW: Memory Leak - 4,136 bytes in 1 blocks are still reachable
         s = alcGetString(nullptr, ALC_ALL_DEVICES_SPECIFIER);
     } else {
         s = alcGetString(nullptr, ALC_DEVICE_SPECIFIER);
@@ -873,8 +889,12 @@ bool SGSoundMgr::testForError(std::string s, std::string name)
 #ifdef ENABLE_SOUND
     ALenum error = alGetError();
     if (error != AL_NO_ERROR)  {
-       SG_LOG( SG_SOUND, SG_ALERT, "AL Error (" << name << "): "
-                                      << alGetString(error) << " at " << s);
+        const auto errMsg = alGetString(error);
+        if (!errMsg) {
+            SG_LOG(SG_SOUND, SG_ALERT, "AL error (" << name << ") " << error << " (no string defined) at " << s);
+        } else {
+            SG_LOG(SG_SOUND, SG_ALERT, "AL Error (" << name << "): " << errMsg << " at " << s);
+        }
        return true;
     }
 #endif
